/*
 * DRV5055-5057EVM ADCMeasurement.c
 *
 * The source code for functions related to the ADC that are utilized
 * by the main.c file
 *
 * Copyright (C) 2018 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Patrick Simmons, a0226740
 * Texas Instruments Inc.
 * March 2018
 * Built with Code Composer Studio v5.5
 */
#include <msp430.h>
#include <DRVGlobalVariables.h>
unsigned char ADCChannel;
//unsigned int ADCENC = 0x02;


ADCDefaultSetup()
{
    ADCCTL0 |= ADCSHT_8 | ADCON;                             // ADCON, S&H=16 ADC clks
    ADCCTL1 |= ADCSHP | ADCDIV_4;                            // ADCCLK = MODOSC; sampling timer
    ADCCTL2 |= ADCRES;                                       // 10-bit conversion results
    ADCMCTL0 |= ADCINCH_0;                                   // A0 ADC input select;
    ADCChannel=ADCINCH_0;                                    // Sets the ADCchannel to the Default corresponding to DRV5055A1
    ADCIE |= ADCIE0;                                         // Enable ADC conv complete interrupt

    PMMCTL0_H = PMMPW_H;                                     // Open PMM Registers for write
    PMMCTL0_L |= PMMREGOFF_L;                                // Regulator turned off when entering LPM3 and LPM4
    __delay_cycles(400);                                     // Delay for reference settling
    __no_operation();
}
DisableADC()
{
    ADCIE &= ~ADCIE0;                                        //ADC interrupt Disabled
    ADCCTL0 &= ~ADCENC;                                      //ADC conversion Disabled
    ADCCTL0 &= ~ADCSC;                                       //Do not start ADC sample and conversion
    ADCCTL0 &= ~ADCON;                                       //Turn off ADC
    ADCMCTL0 &= ~0xFF;                                       //clear selected input channel
}


ADCChannelChange()
{
    i1=0;
    while(ADCCTL1 & ADCBUSY)
    {
        i1=i1+1;
        if(i1>65000)
        {
            ADCCTL0 &= ~ADCENC;
        }
    }
    ADCIE &= ~ADCIE0;                           //ADC interrupt Disabled
    ADCCTL0 &= ~ADCENC;                         //ADC conversion Disabled
    ADCCTL0 &= ~ADCSC;                          //Do not start ADC sample and conversion
    ADCCTL0 &= ~ADCON;                          //Turn off ADC
    ADCMCTL0 &= ~ADCChannel;                    //clear selected input channel
    d0=10;
    dig1(d0); dig2(d0); dig3(d0); dig4(d0);     //Set the LCD to be blank
    sumC = 0;                                   //Clear ADC sample count
    sumVal = 0;                                 //clear ADC sum value
    oldmemval = 0;                              //reset the old memory value
    newMemVal = 0;                              //reset the new memory value
    if(ADCChannel<8)                            //change to 8 for DRV5055-5057EVM#
    {
        ADCChannel = ADCChannel+1;              //increment to the next ADC channel
        ADCMCTL0 |= ADCChannel;                 //select the input channel
    }
    else
    {
        ADCMCTL0 = 0;                           //reset the selected input channel to A0
        ADCChannel = 0;                         //reset ADCChannel variable
    }
    ADCCTL0 |= ADCSC | ADCENC |ADCON;           //enable conversion, sample and conversion, and turn the ADC on
    ADCIE |= ADCIE0;                            //enable ADC interrupts
}

ADCSampInit()                                    //Initiate sampling of ADC
{
    ADCCTL0 |= ADCENC | ADCSC;                   // Sampling and conversion start
    __bis_SR_register(GIE);                      // LPM0, ADC_ISR will force exit
}
DeviceRange()
{
    intermediate=22*.06/3.3*1023+1; Bound[1]=intermediate;  //calculate bound for DRV5055A1 device Range
    intermediate=44*.03/3.3*1023+1; Bound[2]=intermediate;  //calculate bound for DRV5055A2 device Range
    intermediate=88*.015/3.3*1023+1; Bound[3]=intermediate;  //calculate bound for DRV5055A3 device Range
    intermediate=176*.0075/3.3*1023+1; Bound[4]=intermediate;  //calculate bound for DRV5055A41 device Range


    intermediate=19*.12/3.3*1023+1; Bound[5]=intermediate;  //calculate bound for DRV5056A1 device Range
    intermediate=39*.06/3.3*1023+1; Bound[6]=intermediate;  //calculate bound for DRV5056A2 device Range
    intermediate=78*.03/3.3*1023+1; Bound[7]=intermediate;  //calculate bound for DRV5056A3 device Range
    intermediate=155*.015/3.3*1023+1; Bound[8]=intermediate;  //calculate bound for DRV5056A4 device Range
    //Sensitivity[0] = 0;
    Sensitivity[1]= 60; Sensitivity[2]=30; Sensitivity[3]= 15; Sensitivity[4]= 7.5;  //Define sensitivity of DRV5055 gain variants at 3.3V VCC
    Sensitivity[5]= 120; Sensitivity[6]= 60; Sensitivity[7]= 30; Sensitivity[8]= 15;  //Define sensitivity of DRV5056 gain variants at 3.3V VCC
}

