/*
 * DRV5055-5057EVM ClocksTimers.c
 *
 * The source code for functions related to the Clocks that are utilized
 * by the main.c file
 *
 * Copyright (C) 2018 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Patrick Simmons, a0226740
 * Texas Instruments Inc.
 * March 2018
 * Built with Code Composer Studio v5.5
 */
#include <msp430.h>
#include <DRVGlobalVariables.h>

CLKSetup()
{
    // Configure clock
    FRCTL0 = FRCTLPW | NWAITS0;                     // Set number of FRAM waitstates to 0
    __bis_SR_register(SCG0);                        // disable FLL
    CSCTL3 |= SELREF__REFOCLK;                      // Set REFO as FLL reference source
    CSCTL0 = 0;                                     // clear DCO and MOD registers
    CSCTL1 &= ~(DCORSEL_7);                         // Clear DCO frequency select bits first
    CSCTL1 |= DCORSEL_3;                            // Set DCO = 8MHz
    CSCTL2 = FLLD_0 + 243;                          // FDCOCLK/1, DCODIV = 8MHz
                                                    //FDCOCLK=(2^FLLD)*(FLLN/1)*(FLLREFCLK/n)
                                                    //FDCOCLK=(2^0)*(243/1)*(32768/1)=7,962,624Hz
    __delay_cycles(3);
    __bic_SR_register(SCG0);                        // enable FLL
    while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1));      // Poll until FLL is locked
    __delay_cycles(3);
    CSCTL4 = SELMS__DCOCLKDIV | SELA__REFOCLK;      // set default REFO(~32768Hz) as ACLK source, ACLK = 32768Hz
                                                    // default DCODIV as MCLK and SMCLK source
    CSCTL5 |= DIVM__1 | DIVS__4;                    // MCLK=DCODIV/1=8MHZ, SMCLK = MCLK/4=2MHz
}

TimerSetup()
{
    TA1CCTL2 |= CM_1 | CCIS_0 | CAP | SCS|CCIE;     // Capture rising edge, Use CCI2A=ACLK, Synchronous capture, Enable capture mode, Enable capture interrupt
    TA1CTL |= TASSEL_2 | MC_2 | TACLR;              // Use SMCLK as clock source, clear TA0R, Start timer in continuous mode
    j=1<<Psamples;                                  // variable that is used for setting our REC and FEC indexes
}
