/*
 * DRV5055-5057EVM LCDSetup.c
 *
 * The source code for functions related to the LCD that are utilized
 * by the main.c file
 *
 * Copyright (C) 2018 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Patrick Simmons, a0226740
 * Texas Instruments Inc.
 * March 2018
 * Built with Code Composer Studio v5.5
 */
#include <msp430.h>
#include <DRVGlobalVariables.h>
LCDSetup()
{
        SYSCFG2 |= LCDPCTL;                                                //
        LCDPCTL0 |= 0xFF3F;                                                //L0-L5 and L8-L15 pins selected for LCD function
        LCDPCTL1 |= 0xEF3F;                                                //L16-L21 and L24-L31 pins selected for LCD function
        LCDPCTL2 |= 0x0033;                                                // L32,L33,L36,L37 pins selected for LCD function
        LCDCTL0 = LCDSSEL_1 | LCDDIV_7;                                    // fLCD ref=fsource=ACLK=32768Hz, fLCD=fsource/(LCDDIVx+1)*value[LCDMXx]
                                                                           //fLCD = 32768/((7+1)*64)=64Hz;
                                                                           //f_Charge_pump = 256Hz, Internal Vref = 2.96V, Mode 3
        LCDVCTL = LCDCPEN | LCDREFEN | VLCD_6 | (LCDCPFSEL0 | LCDCPFSEL1 | LCDCPFSEL2 | LCDCPFSEL3);
        LCDMEMCTL |= LCDCLRM;                                              // Clear LCD memory
        LCDCSSEL0 = BIT5;                                                  // L5 COM pin, all others Segments
        LCDCSSEL1 = 0x0000;                                                // pins L16-L31 set to be segments
        LCDCSSEL2 = 0x0000;                                                // pins L32-L47 set to be segments
        LCDM2 = 0x10;                                                      //L5 = COM0


        LCDCTL0=0;                                                         //Clear this register to remove any previous default values
        LCDCTL0 |= LCDDIV_13 | LCDSSEL_1 | LCDSTATIC | LCDSON | LCDON;     //Divide by 13, clock source =ACLK, LCD Mux =000b (Static), enable all LCD segments, turn LCD on
}
dig1(num)
{
    switch(num)
    {
    case 0 : LCDM18=0x11; LCDM16=0x11; LCDM15=0x10; LCDM14=0x10; break;    //"_ _ _ 0"
    case 1 : LCDM18=0x10; LCDM16=0x10; LCDM15=0x00; LCDM14=0x00; break;    //"_ _ _ 1"
    case 2 : LCDM18=0x11; LCDM16=0x00; LCDM15=0x11; LCDM14=0x10; break;    //"_ _ _ 2"
    case 3 : LCDM18=0x11; LCDM16=0x10; LCDM15=0x11; LCDM14=0x00; break;    //"_ _ _ 3"
    case 4 : LCDM18=0x10; LCDM16=0x11; LCDM15=0x01; LCDM14=0x00; break;    //"_ _ _ 4"
    case 5 : LCDM18=0x01; LCDM16=0x11; LCDM15=0x11; LCDM14=0x00; break;    //"_ _ _ 5"
    case 6 : LCDM18=0x01; LCDM16=0x11; LCDM15=0x11; LCDM14=0x10; break;    //"_ _ _ 6"
    case 7 : LCDM18=0x11; LCDM16=0x10; LCDM15=0x00; LCDM14=0x00; break;    //"_ _ _ 7"
    case 8 : LCDM18=0x11; LCDM16=0x11; LCDM15=0x11; LCDM14=0x10; break;    //"_ _ _ 8"
    case 9 : LCDM18=0x11; LCDM16=0x11; LCDM15=0x01; LCDM14=0x00; break;    //"_ _ _ 9"
    case 10: LCDM18=0x00; LCDM16=0x00; LCDM15=0x00; LCDM14=0x00; break;    //"_ _ _  "
    case 11: LCDM18=0x00; LCDM16=0x00; LCDM15=0x01; LCDM14=0x00;           //"_ _ _ -"
    }
}
dig2(num)
{
    switch(num)
    {
    case 0 : LCDM13=0x11; LCDM12=0x11; LCDM10=0x10; LCDM9=0x10; break;    //"_ _ 0 _"
    case 1 : LCDM13=0x10; LCDM12=0x10; LCDM10=0x00; LCDM9=0x00; break;    //"_ _ 1 _"
    case 2 : LCDM13=0x11; LCDM12=0x00; LCDM10=0x11; LCDM9=0x10; break;    //"_ _ 2 _"
    case 3 : LCDM13=0x11; LCDM12=0x10; LCDM10=0x11; LCDM9=0x00; break;    //"_ _ 3 _"
    case 4 : LCDM13=0x10; LCDM12=0x11; LCDM10=0x01; LCDM9=0x00; break;    //"_ _ 4 _"
    case 5 : LCDM13=0x01; LCDM12=0x11; LCDM10=0x11; LCDM9=0x00; break;    //"_ _ 5 _"
    case 6 : LCDM13=0x01; LCDM12=0x11; LCDM10=0x11; LCDM9=0x10; break;    //"_ _ 6 _"
    case 7 : LCDM13=0x11; LCDM12=0x10; LCDM10=0x00; LCDM9=0x00; break;    //"_ _ 7 _"
    case 8 : LCDM13=0x11; LCDM12=0x11; LCDM10=0x11; LCDM9=0x10; break;    //"_ _ 8 _"
    case 9 : LCDM13=0x11; LCDM12=0x11; LCDM10=0x01; LCDM9=0x00; break;    //"_ _ 9 _"
    case 10: LCDM13=0x00; LCDM12=0x00; LCDM10=0x00; LCDM9=0x00; break;    //"_ _   _"
    case 11: LCDM13=0x00; LCDM12=0x00; LCDM10=0x01; LCDM9=0x00;           //"_ _ - _"
    }
}
dig3(num)
{
    switch(num)
    {
    case 0 : LCDM9|=0x01; LCDM8=0x01; LCDM7=0x11; LCDM6=0x10; LCDM5=0x10; break;    //"_ 0 _ _"
    case 1 : LCDM9|=0x01; LCDM8=0x00; LCDM7=0x10; LCDM6=0x00; LCDM5=0x00; break;    //"_ 1 _ _"
    case 2 : LCDM9|=0x01; LCDM8=0x01; LCDM7=0x00; LCDM6=0x11; LCDM5=0x10; break;    //"_ 2 _ _"
    case 3 : LCDM9|=0x01; LCDM8=0x01; LCDM7=0x10; LCDM6=0x11; LCDM5=0x00; break;    //"_ 3 _ _"
    case 4 : LCDM9|=0x01; LCDM8=0x00; LCDM7=0x11; LCDM6=0x01; LCDM5=0x00; break;    //"_ 4 _ _"
    case 5 : LCDM9|=0x00; LCDM8=0x01; LCDM7=0x11; LCDM6=0x11; LCDM5=0x00; break;    //"_ 5 _ _"
    case 6 : LCDM9|=0x00; LCDM8=0x01; LCDM7=0x11; LCDM6=0x11; LCDM5=0x10; break;    //"_ 6 _ _"
    case 7 : LCDM9|=0x01; LCDM8=0x01; LCDM7=0x10; LCDM6=0x00; LCDM5=0x00; break;    //"_ 7 _ _"
    case 8 : LCDM9|=0x01; LCDM8=0x01; LCDM7=0x11; LCDM6=0x11; LCDM5=0x10; break;    //"_ 8 _ _"
    case 9 : LCDM9|=0x01; LCDM8=0x01; LCDM7=0x11; LCDM6=0x01; LCDM5=0x00; break;    //"_ 9 _ _"
    case 10: LCDM9|=0x00; LCDM8=0x00; LCDM7=0x00; LCDM6=0x00; LCDM5=0x00; break;    //"_   _ _"
    case 11: LCDM9|=0x00; LCDM8=0x00; LCDM7=0x00; LCDM6=0x01; LCDM5=0x00;           //"_ - _ _"
    }
}
dig4(num)
{
    LCDM2 &= ~0x01;
    switch(num)
    {
    case 0 : LCDM5|=0x01; LCDM4=0x01; LCDM2|=0x01; LCDM1=0x01; LCDM0=0x11; break;    //"0 _ _ _"
    case 1 : LCDM5|=0x01; LCDM4=0x00; LCDM2|=0x00; LCDM1=0x00; LCDM0=0x01; break;    //"1 _ _ _"
    case 2 : LCDM5|=0x01; LCDM4=0x01; LCDM2|=0x01; LCDM1=0x11; LCDM0=0x00; break;    //"2 _ _ _"
    case 3 : LCDM5|=0x01; LCDM4=0x01; LCDM2|=0x00; LCDM1=0x11; LCDM0=0x01; break;    //"3 _ _ _"
    case 4 : LCDM5|=0x01; LCDM4=0x00; LCDM2|=0x00; LCDM1=0x10; LCDM0=0x11; break;    //"4 _ _ _"
    case 5 : LCDM5|=0x00; LCDM4=0x01; LCDM2|=0x00; LCDM1=0x11; LCDM0=0x11; break;    //"5 _ _ _"
    case 6 : LCDM5|=0x00; LCDM4=0x01; LCDM2|=0x01; LCDM1=0x11; LCDM0=0x11; break;    //"6 _ _ _"
    case 7 : LCDM5|=0x01; LCDM4=0x01; LCDM2|=0x00; LCDM1=0x00; LCDM0=0x01; break;    //"7 _ _ _"
    case 8 : LCDM5|=0x01; LCDM4=0x01; LCDM2|=0x01; LCDM1=0x11; LCDM0=0x11; break;    //"8 _ _ _"
    case 9 : LCDM5|=0x01; LCDM4=0x01; LCDM2|=0x00; LCDM1=0x10; LCDM0=0x11; break;    //"9 _ _ _"
    case 10 : LCDM5|=0x00; LCDM4=0x00; LCDM2|=0x00; LCDM1=0x00; LCDM0=0x00; break;    //"  _ _ _"
    case 11 : LCDM5|=0x00; LCDM4=0x00; LCDM2|=0x00; LCDM1=0x10; LCDM0=0x00;           //"- _ _ _"
    }
}
decimal(num)
{
    switch(num)
    {
    case 1 : LCDM4 &= ~0x10; LCDM8 |= 0x10; P4OUT &= ~BIT1; break;    //"_ _ _._"
    case 2 : LCDM4 &= ~0x10; LCDM8 |= 0x10; P4OUT |= BIT1; break;     //"_ _._ _"
    case 3 : LCDM4 |= 0x10; LCDM8 &= ~0x10; break;                    //"_._ _ _"
    case 4 : LCDM4 &= ~0x10; LCDM8 &=~0x10; break;                    //"_ _ _ _"
    }
}

//Function choosing what number to write to all 4 digits when ADC > 999
d4w(c_val)
{
    d3=c_val/1000;                                    //d3 x x x ==> 0 0 0 d3
    d2=(c_val-d3*1000)/100;                           //(d3 d2 x x - d3 0 0 0)/100  ==> 0 0 0 d2
    d1=(c_val-d3*1000-d2*100)/10;                     //(d3 d2 d1 x - d3 d2 0 0)/10  ==> 0 0 0 d1
    d0=(c_val-d3*1000-d2*100-d1*10);                  //(d3 d2 d1 d0 - d3 d2 d1 0)  ==> 0 0 0 d0
}
//Function choosing what number to write to all 4 digits when 999>ADC>99
d3w(c_val)
{
    d3=0;                                              // Leading 0, change to 10 for blank screen
    d2=(c_val)/100;                                    // d2 x x ==> 0 0 0 d2
    d1=(c_val-d2*100)/10;                              // (d2 d1 x - d2 0 0)/10  ==> 0 0 d1
    d0=(c_val-d2*100-d1*10);                           // (d2 d1 d0 - d2 d1 0)  ==> 0 0 d0
}
//Function choosing what number to write to all 4 digits when 99>ADC>9
d2w(c_val)
{
    d3=10;                                              // 0 if blank screen not desired
    d2=0;                                               // 10 if blank screen desired
    d1=(c_val)/10;
    d0=(c_val-d1*10);
}
//Function choosing what number to write to all 4 digits when 9>ADC
d1w(c_val)
{
    d3=10;                                              // set to 0 if blank screen not desired
    d2=10;                                              // set to 0 if blank screen not desired
    d1=0;                                               // set to 10 if blank screen desired
    d0=c_val;
}

//Function for converting ADCMEM value to voltage value
M2V(c_val)
{
    V_Val = c_val*3.3*0.000976563;
}

LCDOUT()
{

    if(Device<9)                                        //The following corresponds to the DRV5055, 56 devices
    {
        if(mode==1)                                     //mode1=ADC% LCD display
        {
            if (oldmemval > 999) {d4w(oldmemval);}      //if ADC memory value greater than 999, call d4w method
            else if (oldmemval >99){d3w(oldmemval);}    //if ADC memory value greater than 99, call d3w method
            else if (oldmemval > 9){d2w(oldmemval);}    //if ADC memory value greater than 9, call d2w method
            else                   {d1w(oldmemval);}    //if ADC memory value less than 9, call d1w method
            dig1(d0); dig2(d1); dig3(d2); dig4(d3);     //Convert d0-d3 to correct bit pattern for LCD
        }
        else if(mode==2)                                //mode2 = Voltage LCD display
        {
            M2V(oldmemval);                             //Call method to convert the ADC value to a voltage
            d3=V_Val;                                   //d3 is an char which will only keep the integer value
            intermediate = d3;                          //intermediate is a double value
            intermediate = V_Val-intermediate;          //double - double = double: (d3 . d2 d1 d0 - d3 . 0  0 0) = 0. d2 d1 d0
            d2=intermediate*10;                         //(char) (d2 . d1 d0) = d2
            d1=intermediate*100-d2*10;                  //(char) d2 d1 . d0 - d2 0 = d1
            d0=intermediate*1000-d2*100-d1*10;          //(char) d2 d1 d0 - d2 d1 0 = d0
            dig1(d0); dig2(d1); dig3(d2); dig4(d3); decimal(3);  //Convert d0-d3 to correct bit pattern for LCD, and include the front decimal
        }
        else if(mode==3)                                //mode3 = mT LCD display
        {
            range[0]=centerVals[Device]-Bound[Device];  //Calculate range lower limit
            range[1]=centerVals[Device]+Bound[Device];  //Calculate range upper limit
            if(Device > 4){range[0]=centerVals[Device];}//Set lower limit to "centerVals" because DRV5056 devices are unipolar
            if(oldmemval < centerVals[Device])          //Is the measured value lower than when no magnetic stimulus is applied
            {
                if(oldmemval < range[0])  {ErrC=1;}     //check if we are beneath the linear range, and make ErrC = 1 if so
                else {ErrC=0;}                          //We are operating in the linear range
                direction=0;                            //value that ensures a negative sign is displayed on the LCD
                diff=centerVals[Device]-oldmemval;      //difference in current measured value and the value when no magnetic stimulus is present
            }
            else if(oldmemval >= centerVals[Device])    //Is the measured value higher than when no magnetic stimulus is applied
            {
                if(oldmemval > range[1])  {ErrC=1;}     //check if we are above the linear range, and make ErrC = 1 if so
                else {ErrC=0;}
                direction=1;                            //value that ensures a no sign is displayed on the LCD
                diff=oldmemval-centerVals[Device];      //difference in current measured value and the value when no magnetic stimulus is present
            }
            if(ErrC==1)                                              //To enter this if-statement, the measurement is outside of the linear range
            {
                dig1(11); dig2(11); dig3(11); dig4(11); decimal(4);  //This makes the LCD display "- - - -"
            }
            else
            {
                M2V(diff);                               //Convert the difference between measured magnetic field and no magnetic field in memory to Voltage value
                mTVal=V_Val*1000/Sensitivity[Device];    //This converts the voltage value to mV, then divides by the sensitivity to get the mT value
                mTOUT();                                 //This calls the function converts the mT value to the corresponding bit pattern for the LCD

            }
        }
    }
}

PWMLCDOUT()                                              //The following corresponds to the DRV5057 devices
{
    if(mode==1)
    {
        Duty2LCDDig();                                   //Calls the method to convert the calculated duty cycle into the correct bit pattern to display on the LCD
    }
    else if(mode==2)                                     //mode2 = Voltage LCD display
    {
        d0=10;                                           //10 is for the blank screen
        dig1(d0); dig2(d0); dig3(d0); dig4(d0); decimal(4);  //PWM could be converted to a voltage, but we chose not to...
    }
    else
    {
        Prange[0]=PWMcVals[Device-9]-PWMBound[Device-9];  //Calculate range lower limit
        Prange[1]=PWMcVals[Device-9]+PWMBound[Device-9];  //Calculate range upper limit
        if(Dave<PWMcVals[Device-9])                       //Is the measured value lower than when no magnetic stimulus is applied
        {
            if(Dave < Prange[0])  {ErrC=1;}               //check if we are beneath the linear range, and make ErrC = 1 if so
            else {ErrC=0;}
            direction=0;                                  //value that ensures a negative sign is displayed on the LCD
            Pdiff=PWMcVals[Device-9]-Dave;                //difference in current measured value and the value when no magnetic stimulus is present

        }
        else if(Dave>=PWMcVals[Device-9])                 //Is the measured value higher than when no magnetic stimulus is applied
        {
            if(Dave > Prange[1])  {ErrC=1;}                //check if we are above the linear range, and make ErrC = 1 if so
            else {ErrC=0;}
            direction=1;                                   //value that ensures a no sign is displayed on the LCD
            Pdiff=Dave-PWMcVals[Device-9];                 //difference in current measured value and the value when no magnetic stimulus is present
        }

        if(ErrC==1)                                        //To enter this if-statement, the measurement is outside of the linear range
        {
            dig1(11); dig2(11); dig3(11); dig4(11); decimal(4);   //This makes the LCD display "- - - -"
        }
        else
        {
            mTVal=Pdiff/PWMSensitivity[Device-9];           //the device is in the linear range, convert the difference to a mT value
            mTOUT();                                        //call the method that converts the mT val to the correct LCD bit pattern
        }
    }
}
mTOUT()
{
        if(mTVal<100)
        {
            d3=mTVal/10;                                        //(char) (d3 d2 . d1 d0)/10 = d3
            d2=mTVal-d3*10;                                     //(char) (d3 d2 . d1 d0 - d3 0) = d2
            d1=(mTVal-d3*10-d2)*10;                             //(char) ((d3 d2 . d1 d0 - d3 d2)*10) =d1
            intermediate=(mTVal-d3*10-d2)*100;                  //intermediate = d1 d0
            intermediate2=d1*10;                                //
            d0=intermediate-d1*10;                              // d1 d0 - d1 0 = d0

            /*if(d0>11)
            {
                mode=1;
            }*/
            if(mTVal<10){d3=11-direction; i=2;}                                //if mT measurement under 10 mT, make d3 the sign digit
            else if(mTVal<100){d0=d1; d1=d2; d2=d3; d3=11-direction; i=1;}     //if measurement greater than 10 mT, shift all the digits rightward
            dig1(d0); dig2(d1); dig3(d2);                                      //Convert numbers to correct bit pattern to display on the LCD
            dig4(d3); decimal(i);
        }

        else if(mTVal>100)                                      //probably need to modify
        {
            d3=11-direction;                                    //d3= "-" if negative and " " if positive
            d2=mTVal/100;                                       //(char) (d2 d1 d0)/100 = d2
            d1=(mTVal-d2*100)/10;                               //(char) (d2 d1 d0 - d2 0 0)/10 = d1
            d0=(mTVal-d2*100-d1*10);                            //(char) (d2 d1 d0 - d2 d1 0) = d0
            dig1(d0); dig2(d1); dig3(d2);                       //Convert numbers to correct bit pattern to display on the LCD
            dig4(d3); decimal(4);                               //
        }
        else
        {
            //Do nothing as this should not be an option
        }

}
