/*
 * DRV5055-5057EVM PWMMeasurement.c
 *
 * The source code for functions related to measurements of the PWM
 * signals generated by the DRV5057 devices.
 * by the main.c file
 *
 * Copyright (C) 2018 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Patrick Simmons, a0226740
 * Texas Instruments Inc.
 * March 2018
 * Built with Code Composer Studio v5.5
 */
#include <msp430.h>
#include <DRVGlobalVariables.h>

PWMSetup()                                             // Configure Channel Select GPIOs to CD74HC153 and timer Capture Pin
{
       P4DIR |= BIT2 | BIT0;                           //BIT2= TIM_S0
       P8DIR |= BIT2;                                  //BIT2=TIM_S1
       P4OUT &= ~(BIT2|BIT1);                          //P4.2 Low for TIM_S0
       P8OUT &= ~BIT2;                                 //P8.2 Low for TIM_S1
       P8SEL0 |= BIT3;                                 //TA1.CCI1A input capture pin, second function
       P8REN |= BIT3;                                  //enable P8.3 pull-up resistor
       P8OUT |= BIT3;                                  //enable P8.3 pull-up resistor
}

PWMDRange()
{
    intermediate=263*.15; PWMBound[3]=intermediate;    //Calculate the Boundary value for DRV5057A4 at 3.3V
    intermediate=131*.3; PWMBound[2]=intermediate;     //Calculate the Boundary value for DRV5057A3 at 3.3V
    intermediate=66*.6; PWMBound[1]=intermediate;      //Calculate the Boundary value for DRV5057A2 at 3.3V
    intermediate=33*1.2; PWMBound[0]=intermediate;     //Calculate the Boundary value for DRV5057A1 at 3.3V

    PWMSensitivity[3]= 0.15; PWMSensitivity[2]=0.30; PWMSensitivity[1]= 0.60; PWMSensitivity[0]= 1.20;  //Defining the sensitivity at 3.3V for each of the gain variants
}

DisablePWM()                                          //Stops PWM from interfering with the rest of the program, by removing its ability to trigger an interrupt
{
    TA1CCTL2 &= ~CCIE;                                //timer A1.2 capture/compare enable disabled
    TA1CCTL2 &= ~CCIFG;                               //timer A1.2 capture/compare interrupt flag cleared
    P4OUT |= BIT0;                                    //Make sure no PWM signal reaches the Microcontroller as it can interfere with the I2C
    TA1CTL |= TACLR;                                  //time A1 clear and set TA1R to 0
}


PWMChannelSel()
{
    switch(Device)                                     //Flip on the Appropriate channels of the CD74HC153
    {
    case 9 : P4OUT &= ~BIT2; P8OUT &= ~BIT2; break;    //Turn on 2I0 to 57A1
    case 10: P4OUT |= BIT2; P8OUT &= ~BIT2; break;     //Turn on 2I3 to 57A2
    case 11: P4OUT &= ~BIT2; P8OUT |= BIT2; break;     //Turn on 2I2 to 57A3
    case 12: P4OUT |= BIT2; P8OUT |= BIT2; break;      //Turn on 2I1 to 57A4
    }
}

PWMSampInit()
{
    //P4OUT &= ~BIT0;
     __bis_SR_register(LPM0_bits | GIE);              //enter low power mode 0, enable general interrupts, wait for timer interrupt
    if(state==1){TA1CCTL2 |= CM_2;}                   //set to capture on falling edge
    else{TA1CCTL2 &= ~CM_2;}                          //no capture
}

PWMCanTransition()
{
    DisablePWM();                                     //disable capture/compare, remove interrupt flag, block PWM signal from reaching microcontroller, clear timer count register
    PWMChannelSel();                                  //Choose the correct Mux channel
    P4OUT &= ~BIT0;                                   //Allow PWM signals to reach the microcontroller
    TA1CCTL2 |= CCIE;                                 //Enable capture mode
    sumC = 0;                                         //reset variable that keeps track of how many samples have been collected for an average sample
}
PWMCalc()
{
    if(tempVal2)                                      //(count & 0x01)//if (count % 2)
    {
        if(count2>count1)                             //Once count2 is incremented higher than count1, another Rising edge measurement can be taken
        {REC[j-count1]=tempVal; count1++;}            //Array is being filled from last to first, only enter rising edge value if count2 is higher
    }
    else
    {
        if(count2<count1+2)                           //count 2 initially lower than count1+2, so second sample will always be entered into FEC
        {FEC[j+1-count2]=tempVal; count2++;}          //Array is being filled from last to first,
    }
    //count++;
    if (count1>j)                                     //(2^Psamples)*2-1
    {
        DisablePWM();                                 //disable capture/compare, remove interrupt flag, block PWM signal from reaching microcontroller, clear timer count register
        /*TA1CCTL2 &= ~(CCIE);
        P4OUT |= BIT0;
        TA1CTL |= TACLR;*/
        Dsum=0;                                       //Reset the value for the Duty Cycle sample summation
        for(i=j; i>0; i--)
        {
            DutyLow=REC[i-1]-FEC[i];                  //This calculates the difference between the falling edge and preceding rising edge
            Period=REC[i-1]-REC[i];                   //This calculates the period between the rising edge and preceding rising edge

            duty=(1-DutyLow/Period)*100;              //This calculates the duty cycle
            //DArr[i]=duty;                           //An array meant for debug and monitoring sample variance jitter
            Dsum= Dsum+duty;                          //this sums up the duty cycle values for averaging later
        }
        Psamples2=1<<Psamples;                        //This corresponds to how many samples were taken for the average
        Dave= Dsum/Psamples2;                         //This calculates the average of duty cycle

        //mode=1;
        if (SetupComplete == 1)
       {
            if(Dave>=0)                               //As long as the calculated average is not negative, initiate steps to convert to LCD bit pattern
            {
                PWMLCDOUT();                          //Call the method that determine how the PWM output should be processed
            }
            check=1;                                  //Change the check status variable to indicate that the slider switch status needs to be checked
       }
       else                                           //Collect Calibration Values
       {
           PWMcVals[Device-9]=Dave;                   //Set the Averaged value to be the
           if(Dave<0){check=0;}                       //non negative duty cycle calculated, proceed with measuring next device
           else{check=1;}                             //redo sampling for calibration, a negative duty cycle was calculated
       }

        count1=0;                                     //reset count1 to 0
        count2=0;                                     //reset count2 to -
        TA1CCTL2 |= CCIE;                             //timer A1.2 capture/compare enabled
        P4OUT &= ~BIT0;                               //
        state = 0;                                    //reset state to 0 so that we know when we capture the first rising edge and we satisfy our method of collecting samples
        //check =1;
    }
}

Duty2LCDDig()
{
    i1 = Dave;                                        //Save Double Dave value as an int value
    i2 = i1;
    i3 = (Dave - i2)*100;                             //(d3 d2 . d1 d0 - d3 d2 ) = d1 d0
    d3=Dave/10;                                       //(char) (d3 d2 . d1 d0)/10 = d3
    d2=(Dave-d3*10);                                  //(char) (d3 d2 . d1 d0 - d3 0) = d2
    d1=i3/10;                                         //(d1 d0)/10 = d1
    d0=(i3-d1*10);                                    //(d1 d0 - d1 0 ) = d0
    dig1(d0); dig2(d1); dig3(d2); dig4(d3); decimal(2); //Convert d0-d3 to correct bit pattern for LCD, and include the middle decimal
}



