/*
 * DRV5055-5057EVM PushButtonSwitch.c
 *
 * The source code for functions related to the Pushbutton that are utilized
 * by the main.c file
 *
 * Copyright (C) 2018 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Patrick Simmons, a0226740
 * Texas Instruments Inc.
 * March 2018
 * Built with Code Composer Studio v5.5
 */

#include <msp430.h>
#include <DRVGlobalVariables.h>

PBSetup()
{
        P4DIR |= BIT1;                               //set P4.1 as an output
        P4REN |= BIT1;                               //enable pullup resistor for P4.1****
        P4OUT |= BIT0;                               //Make sure no PWM signal reaches the Microcontroller as it can interfere with the I2C
        P2OUT |= BIT4;                               // Configure P2.4 as pulled-up
        P2REN |= BIT4;                               // P2.4 pull-up register enable
        P2IES |= BIT4;                               // P2.4 Hi/Low edge
        P2IE |= BIT4;                                // P2.4 interrupt enabled
        P2IFG &= ~BIT4;                              // P2.4 IFG cleared
        P5SEL0 |= BIT2 | BIT3;                       // Configure Pins P5.2 and P5.3 for I2C
        __bis_SR_register(GIE);                      // Enable General Interrupts
}

TXDataLoader(Device)                                  // Configure USCI_B0 for I2C mode
{
    __bic_SR_register(GIE);
    UCB0CTLW0 |= UCSWRST;                             // put eUSCI_B in reset state
    UCB0CTLW0 |= UCMODE_3 | UCMST | UCSYNC;           //UART MODE with Automatic Baud rate detection, master mode, synchronous mode
    UCB0CTLW1 |= UCASTP_2;                            //stop generated automatically after byte counter value reached
    UCB0TBCNT = 0x0002;                               //number of bytes to be received
    if(Device != 17)
    {
        UCB0CTLW0 &= ~UCSYNC;                          //Disable Synchronous Mode
        UCB0CTLW1 &= ~UCASTP_2;                        //Disable Automatic stop
        UCB0TBCNT &= ~0x0002;                          //clear UCB0TBCNT
    }
    UCB0BRW = 0x82;                                    // baudrate = ################
    UCB0CTLW0 &=~ UCSWRST;                             // clear reset register
    UCB0IE |= UCTXIE0 | UCNACKIE | UCRXIE | UCBCNTIE;  // transmit, NACK, Recieve, and Byte Counter interrupt enabled
    UCB0I2CSA = 0x20;                                  // ######
    while (UCB0CTLW0 & UCTXSTP);{}                     // Microcontroller needs extra time to setup registers correctly
    TXByteCtr = 2;                                     // default condition for cases 1 through 16
    RBit = 0;                                          // used for debug
    switch(Device)
    {
    case 1: {TXData[0]= 0x03; TXData[1]=0xFE; break; } //DRV5055A1
    case 2: {TXData[0]= 0x03; TXData[1]=0xFD; break; } //DRV5055A2
    case 3: {TXData[0]= 0x03; TXData[1]=0xFB; break; } //DRV5055A3
    case 4: {TXData[0]= 0x03; TXData[1]=0xF7; break; } //DRV5055A4
    case 5: {TXData[0]= 0x03; TXData[1]=0xEF; break; } //DRV5056A1
    case 6: {TXData[0]= 0x03; TXData[1]=0xDF; break; } //DRV5056A2
    case 7: {TXData[0]= 0x03; TXData[1]=0xBF; break; } //DRV5056A3
    case 8: {TXData[0]= 0x02; TXData[1]=0xF7; break; } //DRV5056A4
    case 9: {TXData[0]= 0x02; TXData[1]=0xEF; break; } //DRV5057A1
    case 10:{TXData[0]= 0x02; TXData[1]=0xDF; break; } //DRV5057A2
    case 11:{TXData[0]= 0x02; TXData[1]=0xBF; break; } //DRV5057A3
    case 12:{TXData[0]= 0x02; TXData[1]=0x7F; break; } //DRV5057A4
    case 13:{TXData[0]= 0x06; TXData[1]=0x07; break; } //Configure TCA9535 Port 0
    case 14:{TXData[0]= 0x07; TXData[1]=0x80; break; } //Configure TCA9535 Port 1
    case 15: {TXData[0]= 0x02; TXData[1]=0xFF; break; }//Clear Port 0 Output
    case 16: {TXData[0]= 0x03; TXData[1]=0xFF; break; }//Clear Port 1 Output
    case 17:                                           //Read TCA9535 Ports 0 and 1
        {
            TXByteCtr= 1;                              //Used to ensure that the alternate if statement is used in the ISR
            TXData[0]= 0x00; TXData[1]= 0x00;
            UCB0CTLW0 |= UCTR | UCTXSTT;               // I2C TX, start condition
            UCB0TXBUF = TXData[1];                     //Request to read TCA9535 pins P00 and P01
            __bis_SR_register(LPM0_bits | GIE);        // Enter LPM0 w/ interrupts
            UCB0TBCNT = 0x0001;                        //set byte counter to count to 1 before automatically stopping I2C transmission
            UCB0CTLW0 &= ~UCTR;                        //Set master to receiver
            UCB0IE &= ~UCTXIE0;                        //Disable the Transmission Interrupt Enable
            UCB0CTLW0 |= UCTXSTT;                      // I2C start condition
            __bis_SR_register(LPM0_bits|GIE);          // Enter LPM0 w/ interrupt
            RBit=1;                                    //used for debug
            break;
        }
    }
    if (RBit == 0)                                     //used for debug
    {
        UCB0CTLW0 |= UCTR | UCTXSTT;                   //used for debug
        __bis_SR_register(LPM0_bits | GIE);            // Enter Low Power Mode 0 and enable general interrupt
    }

}

