/*
 * DRV5055-5057EVM main.c
 *
 * The source code that takes measurements of a selected DRV5055 device output, processes the measurements,
 * converts the measurements to values corresponding to a format selected by the user, and lastly translates those
 * values into a corresponding output that makes the on board LCD display the proper measurement.
 *
 * Copyright (C) 2018 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *******************************************************************************
 *
 *                              CODE DISCLAIMER
 *
 * This code demonstrates the operation of the DRV5055EVM. For this, the code
 * may rely on the device's power-on default register values and settings such
 * as the clock configuration and care must be taken when combining this code
 * with other independently developed code to avoid potential side effects.
 *
 * --/COPYRIGHT--*/
//******************************************************************************
//                MSP430FR4133
//             -----------------------
// COM|-------|P7.5/L5        P3.0/L8 |-------|4A
//  4E|-------|P7.4/L4        P3.1/L9 |-------|DP3
//  4G|-------|P7.3/L3        P3.2/L10|-------|4B
//  4D|-------|P7.2/L2        P3.3/L11|-------|3E
//  4F|-------|P7.1/L1        P3.4/L12|-------|3G
//  4C|-------|P7.0/L0        P3.5/L13|-------|3D
//  ---0.1uF--|P4.7/R13       P3.6/L14|-------|3F
//  |--0.1uF--|P4.6/R23       P3.7/L15|-------|3C
//  |--0.1uF--|P4.5/R33       P6.0/L16|-------|3A
//  |  -0.1uF-|P4.4/LCDCAP1   P6.1/L17|-------|DP2
//  |  |------|P4.3/LCDCAP0   P6.2/L18|-------|3B
//  |TIM_S0|--|P4.2           P6.3/L19|-------|2E
//  |         |P4.1           P6.4/L20|-------|2G
// GND|-------|DVSS           P6.5/L21|-------|2D
//3.3V|-------|DVCC           P2.0/L24|-------|2F
// RST|-------|RST            P2.1/L25|-------|2C
//TEST|-------|TEST           P2.2/L26|-------|2A
// ENB|-------|P4.0           P2.3/L27|-------|2B
//TIM12|------|P8.3/TA1.2     P2.4/L28|-------|Button
//TIM_S1|-----|P8.2/TA1CLK    P2.5/L29|-------|1E
//56A4|-------|P1.7/A7        P2.6/L30|-------|1G
//56A3|-------|P1.6/A6        P2.7/L31|-------|1D
//56A2|-------|P1.5/A5        P5.0/L32|-------|1F
//56A1|-------|P1.4/A4        P5.1/L33|-------|1C
//55A4|-------|P1.3/A3        P5.2/L34|-------|I2C_SDA
//55A3|-------|P1.2/A2        P5.3/L35|-------|I2C_SCL
//55A2|-------|P1.1/A1        P5.4/L36|-------|1A
//55A1|-------|P1.0/A0        P5.5/L37|-------|1B
//
//                  LCD: VI-401-DP-RC-S
//        =====================================
//          digit4   digit3   digit2   digit1
//            ___      ___      ___      ___
//           |___|    |___|    |___|    |___|
//           |___|    |___|    |___|    |___|
//        =====================================
//

//  Patrick Simmons
//  Texas Instruments Inc.
//  Sept 2017
//  Built with Code Composer Studio v5.5
//******************************************************************************
#include <msp430.h>
#include <DRVGlobalVariables.h>

//LCD Functions
int LCDSetup(void);                                                  //function to set registers for LCD switching Frequency and turn on charge pump caps
int d4w(int num);                                                    //Function that extracts each digit value of a memory value  >999
int d3w(int num);                                                    //Function that extracts each digit value of a memory value  >99
int d2w(int num);                                                    //Function that extracts each digit value of a memory value  >9
int d1w(int num);                                                    //Function that extracts each digit value of a memory value  <10
int dig1(int num);                                                   //Function that selects LCD pins will be used to display the number at the 0 digit place
int dig2(int num);                                                   //Function that selects LCD pins will be used to display the number at the 1 digit place
int dig3(int num);                                                   //Function that selects LCD pins will be used to display the number at the 2 digit place
int dig4(int num);                                                   //Function that selects LCD pins will be used to display the number at the 3 digit place



/*ADC Functions
*========================================*/
int ADCDefaultSetup(void);                                           //Function the sets up the ADC registers and default channel to read from
int DisableADC(void);                                                //Function to Disable the ADC
int ADCChannelChange(void);                                          //Function that changes the selected ADC channel and clears the ADCArray
int ADCSampInit(void);                                               //Function that initiates collecting an ADC Sample
int DeviceRange(void);                                               //Function that calculates the device range

/*ADC Variables
*=========================================*/
unsigned int ADC_Result;                                             //Variable for saving the current ADC value, which then can be put into an Array of Values
//unsigned int ADCarray[64];                                         //Debug array for capturing ADC measurement variance
unsigned int centerVals[9];                                          //Array for DRV5055, DRV5056 output values with no magnetic stimulus present
int N_ave = 5;                                                       //number of ADC Samples to Average = 2^N_ave (if>5 need to change data type)
long sumVal = 0;                                                     //Variable that represents the sum of ADC samples for a sample set
unsigned int sumC = 0;                                               //Corresponds to ADC sample count for a set
int oldmemval = 0;                                                   //Corresponds to the Last Average ADC value
int newMemVal = 0;                                                   //Corresponds to the newest Average ADC value
char d0 = 10;                                                        //Extracted number that will be displayed 0 digit place, (10 = blank bit pattern)
char d1 = 10;                                                        //Extracted number that will be displayed 1 digit place
char d2 = 10;                                                        //Extracted number that will be displayed 2 digit place
char d3 = 10;                                                        //Extracted number that will be displayed 3 digit place



double V_Val;                                                        //Voltage Value calculated from average ADC value
double intermediate;                                                 //intermediate variable used for calculations
double intermediate2;                                                //intermediate variable used for calculations
int Bound[9];                                                        // Array of Boundary values for the DRV5055, DRV5056 Analog signals
double PWMBound[4];                                                  // Array of Boundary values for the DRV5057 PWM signals
double Sensitivity[9];                                               //Sensitivity values for DRV5055, DRV5056 devices at 3.3V VCC
double PWMSensitivity[4];                                            //Sensitivity values for DRV5057 devices at 3.3V VCC
double MaxD[4];
double MinD[4];
int ErrC=0;                                                          //Indicator for device operating outside of linear range
int range[2];                                                        //array that holds the upper and lower bounds of the selected (DRV5055,56) device's range
double Prange[2];                                                    //array that holds the upper and lower bounds of the selected (DRV5057) device's range
int diff;                                                            //difference between no stimulus and measured ADC value
double Pdiff;                                                        //difference between no stimulus and measured PWM value
int direction;                                                       //indicator for positive or negative value
double mTVal;                                                        //Holds the calculated mT value




/*PWM Capture Variables
*=========================================*/
unsigned int Psamples=6;
double Psamples2=0;
unsigned int count=0;
unsigned int count1=0;                                               //Counting variable used for filling Arrays REC and FEC
unsigned int count2=0;                                               //Counting variable used for filling Arrays REC and FEC
unsigned int tempVal=0;                                              //Variable used to collect the TA0CCR2 before it changes
unsigned int tempVal2=0;                                             //Variable for capturing status of P8.3
unsigned int FEC[65];                                                //Array of TA0CCR2 values that correspond to the PWM Falling Edges
unsigned int REC[65];                                                //Array of TA0CCR2 values that correspond to the PWM rising Edges
unsigned int state=0;                                                //used to mark whether or not the first REC sample has been collected
unsigned int i = 0;                                                  //Used as an index to point to values in the FEC and REC arrays
unsigned int j = 0;                                                  //variable that is used for setting our REC and FEC indexes
double Period = 0;                                                   //Period duration for a given duty cycle
double DutyLow = 0;                                                  //Low duration of a duty cycle value calculated from FEC and REC values
double duty=0;                                                       //One Period Duty Cycle
//double DArr[65];                                                   //Array of calculated Duty Cycles
unsigned int i1;                                                     //intermediate variable used for calculations
double i2;                                                           //intermediate variable used for calculations
unsigned int i3;                                                     //intermediate variable used for calculations
double Dave;                                                         //Duty Cycle Average
double Dsum;                                                         //Sum of Duty cycles for an average calculation
//long DsumI;
//double DsumIconv;
double PWMcVals[4];


//Pushbutton and Switch Functions
int PBSetup(void);                                                   //Defines the pushbutton pin and designates 5.2 and 5.3 to communicate I2C to the TCA9535
int TXDataLoader(int num);                                           //Loads the I2C Message to be sent to TCA9535 GPIO Expander Slave


//Pushbutton Variables
unsigned char TXData[]= {0x00,0x00};                                 // {I2C Slave Register Address, I2C Data Byte}
char TXByteCtr;                                                      //
unsigned int Device = 1;                                             //default: Device = 1 (DRV5055A1)
unsigned int RBit = 0;                                               //determines whether to take I2C Read Path of Operation
volatile unsigned char RXData[2];                                    //Array Containing the Byte information Read from the TCA9535



//Clock, Timer, PWM Functions
int CLKSetup(void);                                                  //Function that sets the DCO, MCLK, and SMCLK
int TimerSetup(void);                                                //Function to set up the timers
int PWMSetup(void);                                                  //Defines the pin that will read the PWM signal
int PWMSampInit(void);                                               //Initiates the collection of PWM samples
int PWMChannelSel(void);                                             //Function for toggling the mux to the DRV5057 devices
int PWMCanTransition(void);                                          //Function that prepares the processor for measuring the next devices PWM signal
int PWMDRange(void);                                                 //Function that calculates the Range for the DRV5057 devices
int PWMCalc(void);                                                   //Function to calculate PWM or Duty Cycle
int Duty2LCDDig(void);                                               //Function to convert the Duty Cycle into the correct bit pattern for the LCD
int Duty2mTLCD(void);                                                //Function to convert the Duty cycle to mT value
int PWMLCDOUT(void);                                                 //Function for generating the LCD bit patterns for the DRV5057 devices
int DisablePWM(void);                                                //Function that disables the collection of PWM events

//Random Housekeeping Variables
int SetupComplete=0;                                                 //Ensures that the Pushbutton does not change the Device until all default Registers are properly set
int catch=0;                                                         //Used for Debug
int NextC=0;                                                         //Used to indicate processor needs to switch to the next channel/component
int mode=0;
int oldmode=0;
int check = 0;                                                       //debugging variable
int SwitchBits;                                                      //Variable that stores status of slider switch
/**
 * main.c
 */
int main(void)
{

    WDTCTL = WDTPW | WDTHOLD;                                        // stop watchdog timer

    /**======================================================================//
     * Setup GPIOs
     *///====================================================================//
    PM5CTL0 &= ~LOCKLPM5;
    PBSetup();                                                       //Configure GPIO Pins for Pushbutton and I2C Switch
    DisablePWM();                                                    //Used in case the Device enters unknown state, needs to be restarted and PWM influence at startup needs to be removed
    PWMSetup();                                                    //Configure GPIO Pins for PWM Measurement ##
    PM5CTL0 &= ~LOCKLPM5;                                            //unlock I/O pins
    SYSCFG2 |= ADCPCTL0|ADCPCTL1|ADCPCTL2|ADCPCTL3|ADCPCTL4|ADCPCTL5|ADCPCTL6|ADCPCTL7;  //Enable ADCs A0-A7

    /**======================================================================//
     * Configure the clock used for LCD, PWM, I2C to TCA9535
     *///====================================================================//
    CLKSetup();                                        //Setup the Clock for the Timers
    TimerSetup();

    /**======================================================================//
     * Configure TCA9535 GPIO Expander, and Light up default DRV5055A1 LED indicator
     *///====================================================================//
    P4OUT |= BIT0;                                                   //Make sure no PWM signal reaches the Microcontroller as it can interfere with the I2C
    TXDataLoader(13);                                                //Configure TCA9535 Port 0
    __delay_cycles(10000);                                           //add a delay to give sufficient transmission time
    TXDataLoader(14);                                                //Configure TCA9535 Port 1
    __delay_cycles(10000);                                           //add a delay to give sufficient transmission time
    TXDataLoader(1);                                                 //Light Up DRV5055A1 LED indicator First
    __delay_cycles(10000);                                           //add a delay to give sufficient transmission time
    //P4OUT &= ~BIT0;                                                //Allow PWM signals to reach the microcontroller

    /**======================================================================//
     * Configure Registers for LCD operation
     *///====================================================================//
    LCDSetup();

    /**======================================================================//
    * Configure ADC, Default Channel 0: DRV5055A1
    *///=====================================================================//
    ADCDefaultSetup();

    /**======================================================================//
    * Take preliminary Measurements for Calibration and Calculate Gain Boundaries for DRV5055 and DRV5056 devices
    *///=====================================================================//
    Device=1;
    while(Device<9)
    {
        while(check!=1)                                              //Collect samples for DUT until enough samples have been collected for an average sample and the average is recorded
        {
            ADCSampInit();                                           //initiate ADC sampling for DUT
        }
        check = 0;                                                   //reset check variable
        ADCChannelChange();                                          //Change which ADC channel to sample
        Device=Device+1;                                             //increment device to measure next device
    }

    DisableADC();                                                    //Done collecting calibration for Analog Output devices so disable ADC
    DeviceRange();                                                   //Calculate the device ranges for all of the gain variants

    /**======================================================================//
    * Take preliminary Measurements for Calibration and Calculate Gain Boundaries for DRV5055 and DRV5056 devices
    *///====================================================================//
    PWMCanTransition();                                            //Disable PWM collection, change PWM mux channel, reset sample counter##
    while(Device<13)
    {
        while(check!=1)                                              //Collect samples for DUT until enough samples have been collected for an average sample and the average is recorded
        {
            PWMSampInit();                                           //Initiate the PWM sampling
        }
        Device=Device+1;                                            //Change which device to be measured
        PWMCanTransition();                                         //Disable PWM collection, change PWM mux channel, reset sample counter
        check=0;
    }
    TA1CCTL2 &= ~CCIE;
    PWMDRange();

    SetupComplete = 1;                                               //
    ADCDefaultSetup();                                               //
    Device = 1;                                                      //Set the Device to DRV5055A1 as the initial state for the user
    /**======================================================================//
    * Basic Setup and Calibration Complete start taking measurements
    *///====================================================================//
    while(1)
    {
        if (check==1)                                                //Polling for an change in the slider switch state
                {
                    DisablePWM();                                    //This disables the PWM timer and corresponding measurement

                    P4OUT |= BIT0;                                   //Make sure no PWM signal reaches the Microcontroller as it can interfere with the I2C
                    TXDataLoader(17); __delay_cycles(10000);         //Send I2C signal to TCA device to read the status of the slider switch
                    SwitchBits=RXData[0] & 0x07;                     //Get and save the status of the Slider Switch into SwitchBits

                    if(Device>8){P4OUT |= BIT0; TA1CCTL2 |= CCIE;}   //Disable the PWM timer interrupt P4OUT &= ~BIT1;

                    catch=1; oldmode=mode;                           //save the current mode as the old mode

                    if(SwitchBits == 0x06)                           //Set mode for LCD Percentage Output
                    {
                        mode = 1;
                    }
                    else if(SwitchBits == 0x05)                      //Set mode for LCD Voltage Output
                    {
                        mode = 2;
                    }
                    else                                             //Set mode for LCD mT Output
                    {
                        mode = 3;
                    }
                    check = 0;                                       //Reset check to 0 and wait until an at least one averaged sample is collected
                }
        else if (check < 0  || check >1)
        {
            check = 0;                                               //if check mishandled, reset check
        }
        if(NextC==1)                                                 //Pushbutton pressed need to switch to the next channel/component
        {
            DisablePWM();                                            //disable capture/compare, remove interrupt flag, block PWM signal from reaching microcontroller, clear timer count register
            Device=Device+1;                                         //Set new device to measure
            if(Device==8) { TXDataLoader(16); __delay_cycles(10000);}//clear TCA9535 Port 1
            else if(Device>=13)                                       //Modify this to 13 for DRV5055-5057EVM  ##
            {
                Device=1;                                            //We reached the final device in the sequence, loop around to the starting device (DRV5055A1)
                TXDataLoader(15); __delay_cycles(10000);             //clear TCA9535 Port 0
                ADCChannelChange();                                  //Change the ADC channel
            }
            TXDataLoader(Device); __delay_cycles(10000);             //Switch on the appropriate LED indicator for the device to be measured

            if(Device<9)                                             //If the Device is <9 change to the new ADC channel
            {
                ADCChannelChange();                                  //Change the ADC channel
            }
            else if(Device==9)                                       //If the Device is 9 disable the ADC, Enable and setup PWM, Start PWM sampling
            {
                DisableADC();                                        //device 8 was the last analog output, so turn off the ADC as it is not needed for DRV5057 Measurements
                PWMCanTransition();                                  //Disable PWM collection, change PWM mux channel, reset sample counter
                P4OUT &= ~BIT0;
                PWMSampInit();                                       //Initiate sampling of the PWM
            }
            else if(Device<13)                                       //If the Device is above 9 but below 13, you are looking at DRV5057 devices
            {
                PWMCanTransition();                                  //Disable PWM collection, change PWM mux channel, reset sample counter
                P4OUT &= ~BIT0;
                PWMSampInit();                                       //Inititiate PWM sampling
            }
            NextC=0;                                                 //Reset NextC to 0, so the measured device does not change until prompted by the user
        }
        else if (NextC < 0  || NextC > 1)                            //if NextC mishandled, reset NextC
        {
            NextC = 0;
        }
        if(Device<9)                                                 //Continue sampling ADC until an average can be taken
        {
            ADCSampInit();
        }
        else if(Device<13)                                           //Continue PWM sampling until an average can be taken
        {
            P4OUT &= ~BIT0;
            PWMSampInit();
        }
        else                                                         //If exception occurs, catch it
        {
            catch = 1;                                               //For Debug purposes
        }
   }
}



/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>//
                               INTERRUPT Service ROUTINES
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>*/

/*    ISR Table of Contents
 *    1. Port 1 Interrupt Service Routine
 *    2. I2C Interrupt Service Routine
 *    3. ADC Interrupt Service Routine
 *    4. Timer Interrupt Service Routine
 */

/**================================================================//
 * Port 2 interrupt service routine
 *///==============================================================//
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=PORT2_VECTOR
__interrupt void Port_1(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(PORT1_VECTOR))) Port_1 (void)
#else
#error Compiler not supported!
#endif
{
    DisablePWM();                                                    //disable capture/compare, remove interrupt flag, block PWM signal from reaching microcontroller, clear timer count register
    P2IFG &= ~BIT4;                                                  //Clear P2.4 Interrrupt flag for the pushbutton
    if (SetupComplete==1)                                            //only set NextC to 1 if the initial set up has completed
    {
        NextC=1;                                                     //Make NextC equal so the next device can be measured

    }
    catch=catch+1;                                                   //set catch to 1
    __delay_cycles(10000);
    __bic_SR_register_on_exit(LPM0_bits);
    __bis_SR_register(GIE);
}

/**================================================================//
 * I2C Interrupt Service Routine
 *///==============================================================//
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector = USCI_B0_VECTOR
__interrupt void USCIB0_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(USCI_B0_VECTOR))) USCIB0_ISR (void)
#else
#error Compiler not supported!
#endif
{

    switch(__even_in_range(UCB0IV,0x1E))
    {
        case USCI_NONE:          break;                              // Vector 0: No interrupts break;
        case USCI_I2C_UCALIFG:   break;
        case USCI_I2C_UCNACKIFG: {UCB0CTL1 |= UCTXSTT; break;/*Vector 4: NACKIFG break; resend start if NACK*/}
        case USCI_I2C_UCSTTIFG:  break;                              // Vector 6: STTIFG break;
        case USCI_I2C_UCSTPIFG:  {UCB0IFG &= ~UCSTPIFG; break; /* Vector 8: STPIFG break;*/}
        case USCI_I2C_UCRXIFG3:  break;                              // Vector 10: RXIFG3 break;
        case USCI_I2C_UCTXIFG3:  break;                              // Vector 14: TXIFG3 break;
        case USCI_I2C_UCRXIFG2:  break;                              // Vector 16: RXIFG2 break;
        case USCI_I2C_UCTXIFG2:  break;                              // Vector 18: TXIFG2 break;
        case USCI_I2C_UCRXIFG1:  break;                              // Vector 20: RXIFG1 break;
        case USCI_I2C_UCTXIFG1:  break;                              // Vector 22: TXIFG1 break;
        case USCI_I2C_UCRXIFG0:                                      // Vector for RXIFG0
        {
            RXData[0] = UCB0RXBUF;                                   // Get RX data
            UCB0IFG &= ~UCRXIFG;                                     // Clear USCI_B0 TX interrupt flag
            __bic_SR_register_on_exit(LPM0_bits);                    // Exit LPM0break;
            break;
        }
        case USCI_I2C_UCTXIFG0:                                      //Vector for TXIFG0
        {
            if (TXByteCtr>1)                                         // Check TX byte counter
               {
                UCB0TXBUF = TXData[0];                               // Load TX buffer
                TXByteCtr--;                                         // Decrement TX byte counter
               }
            else if (TXByteCtr>0)
               {
                UCB0TXBUF = TXData[1];                               // Load TX buffer
                TXByteCtr--;                                         // decrement the TX Byte Counter
               }
            else
               {
                UCB0CTLW0 |= UCTXSTP;                                // I2C stop condition
                UCB0IFG &= ~UCTXIFG;                                 // Clear USCI_B0 TX int flag
                __bic_SR_register_on_exit(LPM0_bits);                // Exit LPM0
               }
          break;
        }
        case USCI_I2C_UCBCNTIFG: break;                              // Vector 28: BCNTIFG
        case USCI_I2C_UCCLTOIFG: break;                              // Vector 30: clock low timeout
        case USCI_I2C_UCBIT9IFG: break;                              // Vector 32: 9th bit
        default: break;
    }
}

/**================================================================//
 * ADC Interrupt Service Routine
 *///==============================================================//
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=ADC_VECTOR
__interrupt void ADC_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(ADC_VECTOR))) ADC_ISR (void)
#else
#error Compiler not supported!
#endif
{
    switch(__even_in_range(ADCIV,ADCIV_ADCIFG))
    {
        case ADCIV_NONE: break;
        case ADCIV_ADCOVIFG: break;
        case ADCIV_ADCTOVIFG: break;
        case ADCIV_ADCHIIFG:  break;
        case ADCIV_ADCLOIFG:  break;
        case ADCIV_ADCINIFG:  break;
        case ADCIV_ADCIFG:
            ADC_Result = ADCMEM0;
//            ADCarray[sumC] = ADC_Result;                           //Debug array for assessing measurement variance
            if (sumC < (1 << N_ave)-1) {sumVal=sumVal+ADC_Result; sumC=sumC+1;}  //If we have not collected the required number of samples for averaging just add to the current sum
            else                                                     //If we have collected the required number of samples for averaging, go ahead and average
            {
                sumVal=sumVal+ADC_Result;                            //Add the last collected average to the sum
                newMemVal = sumVal >> N_ave; sumC=0; sumVal=0;       //Average Value = (Sum of ADC values)/(2^N_ave), clear sum and ADC  sample count
                check = 1;                                           //set check to 1 so that the Slider switch status can be checked upon leaving the interrupt service routine
                if (oldmemval==newMemVal && d0 != 10 && oldmode == mode) //Compare the Old and New ADC Average Value
               {
                   //do nothing since value did not change
               }
               else
               {
                   if (SetupComplete == 1)                           //Convert the new measurement to the correct LCD bit pattern
                   {

                       oldmemval= newMemVal;                         //save the current value to compare with the future value, in case it is not necessary to update the LCD
                       LCDOUT();

                   }
                   else                                              //for calibration
                   {
                       centerVals[Device]=newMemVal;                 //Record the measured value for when no magnetic stimulus should be present
                   }
               }
           }
            break;
    }
}

// Timer1_A3 CC1-2, TA Interrupt Handler
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector = TIMER1_A1_VECTOR
__interrupt void TIMER1_A1_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(TIMER1_A1_VECTOR))) TIMER1_A1_ISR (void)
#else
#error Compiler not supported!
#endif
{
    switch(__even_in_range(TA1IV,TA1IV_TAIFG))
    {
        case TA1IV_NONE: break;                                     // No interrupt
        case TA1IV_TACCR1: break;                                   // CCR1 not used
        case TA1IV_TACCR2:
            switch(state)
            {
            case 0:
            {
                tempVal=TA1CCR2;                                    //save the TA1CCR2 value immediately in case the duty cycle is near 0
                state=1;                                            //First rising edge captured, change state to now capture both rising and falling edges
                REC[j+1]=tempVal;                                   //place the rising edge time in the REC array
                //count++;
                break;
            }
            case 1:
            {
                tempVal=TA1CCR2;                                   //save the TA1CCR2 value immediately in case the duty cycle is near 0
                tempVal2=P8IN & BIT3;                              //capture P8.3 status
                PWMCalc();                                         //Call method for call remaining samples and calculating the duty cycle
            }
            break;
            }
        case TA1IV_3: break;                                       // reserved
        case TA1IV_4: break;                                       // reserved
        case TA1IV_5: break;                                       // reserved
        case TA1IV_6: break;                                       // reserved
        case TA1IV_TAIFG: break;                                   // overflow
        default: break;
    }
    __bic_SR_register_on_exit(LPM0_bits);
}
